/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.kernel;

import java.nio.ByteBuffer;
import java.nio.DoubleBuffer;
import java.nio.IntBuffer;

public class SharedMemoryAllocator {
    public static final int CAPACITY = 128;

    private IntBuffer valueReferenceBuffer;
    private DoubleBuffer bufferForDouble;
    private IntBuffer bufferForInt;
    private ByteBuffer bufferForByte;

    public SharedMemoryAllocator() {
        this.valueReferenceBuffer = IntBuffer.allocate(CAPACITY);
        this.bufferForInt = IntBuffer.allocate(CAPACITY);
        this.bufferForDouble = DoubleBuffer.allocate(CAPACITY);
        this.bufferForByte = ByteBuffer.allocate(CAPACITY);
    }

    public DoubleBuffer getSpaceForDouble() {
        bufferForDouble.clear();
        return bufferForDouble;
    }

    public IntBuffer getSharedInteger() {
        return bufferForInt;
    }

    public ByteBuffer getSharedByte() {
        return bufferForByte;
    }

    public IntBuffer getSharedValueReference() {
        return valueReferenceBuffer;
    }

    public DoubleBuffer shareDoubles(int numberOfDoubles) {
        return DoubleBuffer.allocate(numberOfDoubles);
    }

    public IntBuffer shareIntegers(int numberOfIntegers) {
        return IntBuffer.allocate(numberOfIntegers);
    }
}
