/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.kernel.operatingsystem;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import static org.javafmi.kernel.operatingsystem.FileUtils.*;

public class Zipper {

    public File zipDirectory(File directory, String zipName) {
        File zipFile = new File(zipName);
        ZipOutputStream out = new ZipOutputStream(fileOutputStream(zipFile));
        addDir(directory, out);
        close(out);
        return zipFile;
    }

    private void addDir(File dirObj, ZipOutputStream out) {
        try {
            for(File file : dirObj.listFiles()) {
                if(file.isDirectory()) {
                    putNextEntry(out, new ZipEntry(file.getPath().substring(file.getPath().indexOf(File.separatorChar) + 1).replace(File.separatorChar, '/') + "/"));
                    addDir(file, out);
                    out.closeEntry();
                    continue;
                }
                String path = file.getPath();
                putNextEntry(out, new ZipEntry(path.substring(path.indexOf(File.separator) + 1).replace(File.separatorChar, '/')));
                BufferedInputStream source = bufferedInputStream(fileInputStream(file));
                copyFromTo(source, out);
                out.closeEntry();
                close(source);
            }
        } catch(IOException e) {
            e.printStackTrace();
        }
    }

    private void putNextEntry(ZipOutputStream out, ZipEntry zipEntry) {
        try {
            out.putNextEntry(zipEntry);
        } catch(IOException e) {
            throw new RuntimeException(e);
        }
    }
}
