/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.kernel.v2;

import org.javafmi.kernel.PendingVariableList;
import org.javafmi.kernel.Status;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static org.javafmi.kernel.Status.ERROR;
import static org.javafmi.kernel.Status.OK;

public class PendingVariablesList {
    private PendingVariableList<Double> pendingReals;
    private Map<Integer, Integer> pendingIntegers;
    private Map<Integer, Boolean> pendingBooleans;
    private Map<Integer, String> pendingStrings;

    public PendingVariablesList() {
        this.pendingReals = new PendingVariableList<>();
        this.pendingIntegers = new HashMap<>();
        this.pendingBooleans = new HashMap<>();
        this.pendingStrings = new HashMap<>();
    }

    public Status setAll(org.javafmi.kernel.FmuProxy proxy) {
        List<Status> statuses = new ArrayList<>();
        statuses.addAll(setPendingReals(proxy));
        statuses.addAll(setPendingIntegers(proxy));
        statuses.addAll(setPendingBooleans(proxy));
        statuses.addAll(setPendingStrings(proxy));
        for (Status status : statuses) {
            if (ERROR == status) return ERROR;
        }
        return OK;
    }

    private List<Status> setPendingReals(org.javafmi.kernel.FmuProxy proxy) {
        List<Status> statuses = new ArrayList<>();
        if(!pendingReals.isEmpty())
            statuses.add(proxy.setReal(pendingReals.getValueReferences(), pendingReals.getValuesAsDoubles()));
        return statuses;
    }

    private List<Status> setPendingIntegers(org.javafmi.kernel.FmuProxy proxy) {
        List<Status> statuses = new ArrayList<>();
        for (Map.Entry<Integer, Integer> entry : pendingIntegers.entrySet())
            statuses.add(proxy.setInteger(entry.getKey(), entry.getValue()));
        return statuses;
    }

    private List<Status> setPendingBooleans(org.javafmi.kernel.FmuProxy proxy) {
        List<Status> statuses = new ArrayList<>();
        for (Map.Entry<Integer, Boolean> entry : pendingBooleans.entrySet())
            statuses.add(proxy.setBoolean(entry.getKey(), entry.getValue()));
        return statuses;
    }

    private List<Status> setPendingStrings(org.javafmi.kernel.FmuProxy proxy) {
        List<Status> statuses = new ArrayList<>();
        for (Map.Entry<Integer, String> entry : pendingStrings.entrySet())
            statuses.add(proxy.setString(entry.getKey(), entry.getValue()));
        return statuses;
    }

    public void addReal(Integer valueReference, Double realValue) {
        pendingReals.add(valueReference, realValue);
    }

    public void addInteger(Integer valueReference, Integer intValue) {
        pendingIntegers.put(valueReference, intValue);
    }

    public void addBoolean(Integer valueReference, Boolean booleanValue) {
        pendingBooleans.put(valueReference, booleanValue);
    }

    public void addString(Integer valueReference, String stringValue) {
        pendingStrings.put(valueReference, stringValue);
    }
}
