/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.kernel.v2.nativeaccess;

import com.sun.jna.*;
import com.sun.jna.ptr.PointerByReference;

public interface FmiV2Interface extends Library {

    Pointer fmi2Instantiate(String modelName, int fmiType, String guID, String resourcesLocation, Structure.ByReference byReference, boolean visible, boolean loggingOn);

    void fmi2FreeInstance(Pointer component);

    int fmi2SetDebugLogging(Pointer component, boolean loggingOn, int numberOfCategories, String[] categories);

    int fmi2SetupExperiment(Pointer component, boolean toleranceDefined, double tolerance, double startTime, boolean stopTimeDefined, double stopTime);

    int fmi2EnterInitializationMode(Pointer component);

    int fmi2ExitInitializationMode(Pointer component);

    int fmi2Terminate(Pointer component);

    int fmi2Reset(Pointer component);

    void fmi2GetReal(Pointer component, int[] valueReferences, int numberOfValueReferences, double[] realValues);

    void fmi2GetInteger(Pointer component, int[] valueReferences, int numberOfValueReferences, int[] intValues);

    void fmi2GetBoolean(Pointer component, int[] valueReferences, int numberOfValueReferences, int[] booleanValue);

    void fmi2GetString(Pointer component, int[] valueReferences, int numberOfValueReferences, String[] stringValues);

    int fmi2SetReal(Pointer component, int[] valueReferences, int numberOfValueReferences, double[] realValues);

    int fmi2SetInteger(Pointer component, int[] valueReferences, int numberOfValueReferences, int[] intValues);

    int fmi2SetBoolean(Pointer component, int[] valueReferences, int numberOfValueReferences, int[] booleanValues);

    int fmi2SetString(Pointer component, int[] valueReferences, int numberOfValueReferences, String[] stringValues);

    int fmi2GetFMUstate(Pointer component, PointerByReference pointerToState);

    int fmi2SetFMUstate(Pointer component, Pointer state);

    int fmi2FreeFMUstate(Pointer component, PointerByReference pointerToState);

    void fmi2SerializedFMUstateSize(Pointer component, Pointer state, Memory stateSize);

    int fmi2SerializeFMUstate(Pointer component, Pointer state, byte[] serializedState, int stateSize);

    int fmi2DeSerializeFMUstate(Pointer component, byte[] serializedState, int stateSize, PointerByReference pointerToState);

    void fmi2GetDirectionalDerivative(Pointer component, int[] unknownVariablesValueReferences, int numberOfUnknowns, int[] knownVariablesValueReferences, int numberOfKnowns, double[] knownDifferential, double[] unknownDifferential);

    int fmi2SetRealInputDerivatives(Pointer component, int[] valueReferences, int numberOfValueReferences, int[] orders, double[] values);

    int fmi2GetRealOutputDerivatives(Pointer component, int[] valueReferences, int length, int[] orders, double[] values);

    int fmi2DoStep(Pointer component, double currentCommunicationPoint, double communicationStepSize, boolean noSetFMUStatePriorToCurrentPoint);

    int fmi2CancelStep(Pointer component);

    int fmi2GetStatus(Pointer component, int statusKind, Memory status);

    int fmi2GetRealStatus(double[] component, int statusKind, double[] status);

    int fmi2GetIntegerStatus(Pointer component, int statusKind, Memory status);

    int fmi2GetBooleanStatus(Pointer component, int statusKind, Memory status);

    int fmi2GetStringStatus(Pointer component, int statusKind, PointerByReference status);

    interface FmiType {
        static final int fmiCoSimulation = 1;
    }

    interface Logger extends Callback {
        void log(Pointer componentEnvironment, String instanceName, int status, String category, String message, Pointer args);
    }

    interface AllocateMemory extends Callback {
        Pointer allocate(int numberOfObjects, int size);
    }

    interface FreeMemory extends Callback {
        void free(Pointer object);
    }

    public interface StepFinished extends Callback {
        void stepFinished(Pointer componentEnvironment, int status);
    }
}
