/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.v1;

import org.javafmi.Simulation;
import org.javafmi.exceptions.JavaFmiException;
import org.javafmi.kernel.Status;
import org.javafmi.kernel.v1.FmuProxy;
import org.javafmi.modeldescription.FmiVersion;
import org.javafmi.modeldescription.ModelDescription;
import org.javafmi.modeldescription.ScalarVariable;

public class Access implements org.javafmi.Access {

    private final ModelDescription modelDescription;
    private FmuProxy proxy;

    public Access(Simulation simulation) {
        this.proxy = new ProxyRetriever(simulation).proxy;
        this.modelDescription = proxy.getModelDescription();
    }

    @Override
    public FmiVersion getVersion() {
        return proxy.getVersion();
    }

    public void instantiate(double timeout) {
        proxy.instantiate(timeout);
    }

    public Status initialize(double startTime, double stopTime) {
        return proxy.initialize(startTime, stopTime);
    }

    public Status terminate() {
        return proxy.terminate();
    }

    public Status reset() {
        return proxy.reset();
    }

    public void freeInstance() {
        proxy.freeInstance();
    }

    public Status doStep(double simulationTime, double stepSize) {
        return proxy.doStep(simulationTime, stepSize);
    }

    public Status cancelStep() {
        return proxy.cancelStep();
    }

    public Object getEnumeration(ScalarVariable modelVariable) {
        return proxy.getEnumeration(modelVariable);
    }

    public double[] getReal(int[] valueReference) {
        return proxy.getReal(valueReference);
    }

    public int getInteger(Integer valueReference) {
        return proxy.getInteger(valueReference);
    }

    public boolean getBoolean(Integer valueReference) {
        return proxy.getBoolean(valueReference);
    }

    public String getString(Integer valueReference) {
        return proxy.getString(valueReference);
    }

    public Status setReal(int[] valueReference, double[] newValue) {
        return proxy.setReal(valueReference, newValue);
    }

    public Status setInteger(Integer valueReference, Integer newValue) {
        return proxy.setInteger(valueReference, newValue);
    }

    public Status setBoolean(Integer valueReference, Boolean newValue) {
        return proxy.setBoolean(valueReference, newValue);
    }

    public Status setString(Integer valueReference, String newValue) {
        return proxy.setString(valueReference, newValue);
    }

    public boolean isDoingLogging() {
        return proxy.isDoingLogging();
    }

    public Status setIsDoingLogging(boolean isDoingLogging) {
        return proxy.setIsDoingLogging(isDoingLogging);
    }

    public boolean isTerminated() {
        return proxy.isTerminated();
    }

    private class ProxyRetriever extends Simulation {
        private final FmuProxy proxy;

        private ProxyRetriever(Simulation simulation) {
            checkVersion(simulation);
            this.proxy = (FmuProxy) getProxy(simulation);
        }

        private void checkVersion(Simulation simulation) {
            if(getProxy(simulation) instanceof FmuProxy) return;
            throw new JavaFmiException("Access to FMU does not match with Simulation version");
        }
    }
}
